"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.handler = exports.description = exports.command = exports.builder = exports.alias = void 0;
var _path = _interopRequireDefault(require("path"));
var _execa = _interopRequireDefault(require("execa"));
var _terminalLink = _interopRequireDefault(require("terminal-link"));
var _cliHelpers = require("@redwoodjs/cli-helpers");
var _projectConfig = require("@redwoodjs/project-config");
var _lib = require("../../lib");
var _serveApiHandler = require("../serveApiHandler");
const command = exports.command = 'flightcontrol <side>';
const alias = exports.alias = 'fc';
const description = exports.description = 'Build, Migrate, and Serve commands for Flightcontrol deploy';
const builder = yargs => {
  yargs.positional('side', {
    choices: ['api', 'web'],
    description: 'select side to build',
    type: 'string'
  }).option('prisma', {
    description: 'Apply database migrations',
    type: 'boolean',
    default: true
  }).option('serve', {
    description: 'Run server for api in production',
    type: 'boolean',
    default: false
  }).option('data-migrate', {
    description: 'Migrate the data in your database',
    type: 'boolean',
    default: true,
    alias: 'dm'
  }).epilogue(`For more commands, options, and examples, see ${(0, _terminalLink.default)('Redwood CLI Reference', 'https://redwoodjs.com/docs/cli-commands#deploy')}`);
};
exports.builder = builder;
const handler = async ({
  side,
  serve,
  prisma,
  dm: dataMigrate
}) => {
  (0, _cliHelpers.recordTelemetryAttributes)({
    command: 'deploy flightcontrol',
    side,
    prisma,
    dataMigrate,
    serve
  });
  const rwjsPaths = (0, _lib.getPaths)();
  const execaConfig = {
    shell: true,
    stdio: 'inherit',
    cwd: rwjsPaths.base,
    extendEnv: true,
    cleanup: true
  };
  async function runApiCommands() {
    if (serve) {
      console.log('\nStarting api...');
      await (0, _serveApiHandler.apiServerHandler)({
        port: (0, _projectConfig.getConfig)().api?.port || 8911,
        apiRootPath: '/'
      });
    } else {
      console.log('\nBuilding api...');
      _execa.default.sync('yarn rw build api', execaConfig);
      prisma && _execa.default.sync(_path.default.join(rwjsPaths.base, 'node_modules/.bin/prisma'), ['migrate', 'deploy', '--schema', `"${rwjsPaths.api.dbSchema}"`], execaConfig);
      dataMigrate && _execa.default.sync('yarn rw dataMigrate up', execaConfig);
    }
  }
  async function runWebCommands() {
    _execa.default.sync('yarn rw build web', execaConfig);
  }
  if (side === 'api') {
    runApiCommands();
  } else if (side === 'web') {
    console.log('\nBuilding web...');
    runWebCommands();
  } else {
    console.log('Error with arguments provided');
    // you broke something, which should be caught by Yargs
  }
};
exports.handler = handler;