"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard").default;
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.description = exports.command = exports.builder = void 0;
var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/instance/includes"));
var _path = _interopRequireDefault(require("path"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
var _terminalLink = _interopRequireDefault(require("terminal-link"));
var _cliHelpers = require("@redwoodjs/cli-helpers");
var _helpers = require("@redwoodjs/fastify-web/helpers");
var webServerCLIConfig = _interopRequireWildcard(require("@redwoodjs/web-server"));
var _lib = require("../lib");
var _colors = _interopRequireDefault(require("../lib/colors"));
var _serveWebHandler = require("./serveWebHandler");
const command = exports.command = 'serve [side]';
const description = exports.description = 'Run server for api or web in production';
function hasServerFile() {
  const serverFilePath = _path.default.join((0, _lib.getPaths)().api.dist, 'server.js');
  return _fsExtra.default.existsSync(serverFilePath);
}
const builder = async yargs => {
  yargs.usage('usage: $0 <side>').command({
    command: '$0',
    description: 'Run both api and web servers',
    builder: yargs => {
      if (!hasServerFile()) {
        yargs.options({
          port: {
            default: (0, _lib.getConfig)().web?.port || 8910,
            type: 'number',
            alias: 'p'
          },
          socket: {
            type: 'string'
          }
        });
        return;
      }
      yargs.options({
        webPort: {
          default: (0, _lib.getConfig)().web?.port || 8910,
          type: 'number'
        }
      }).options({
        apiPort: {
          default: (0, _lib.getConfig)().api?.port || 8911,
          type: 'number'
        }
      });
    },
    handler: async argv => {
      (0, _cliHelpers.recordTelemetryAttributes)({
        command: 'serve',
        port: argv.port,
        host: argv.host,
        socket: argv.socket
      });

      // Run the server file, if it exists, with web side also
      if (hasServerFile()) {
        const {
          bothServerFileHandler
        } = await import('./serveBothHandler.js');
        await bothServerFileHandler(argv);
      } else if ((0, _lib.getConfig)().experimental?.rsc?.enabled || (0, _lib.getConfig)().experimental?.streamingSsr?.enabled) {
        const {
          bothSsrRscServerHandler
        } = await import('./serveBothHandler.js');
        await bothSsrRscServerHandler(argv);
      } else {
        // Wanted to use the new web-server package here, but can't because
        // of backwards compatibility reasons. With `bothServerHandler` both
        // the web side and the api side run on the same server with the same
        // port. If we use a separate fe server and api server we can't run
        // them on the same port, and so we lose backwards compatibility.
        // TODO: Use @redwoodjs/web-server when we're ok with breaking
        // backwards compatibility.
        const {
          bothServerHandler
        } = await import('./serveBothHandler.js');
        await bothServerHandler(argv);
      }
    }
  }).command({
    command: 'api',
    description: 'Start server for serving only the api',
    builder: yargs => yargs.options({
      port: {
        default: (0, _lib.getConfig)().api?.port || 8911,
        type: 'number',
        alias: 'p'
      },
      socket: {
        type: 'string'
      },
      apiRootPath: {
        alias: ['api-root-path', 'rootPath', 'root-path'],
        default: '/',
        type: 'string',
        desc: 'Root path where your api functions are served',
        coerce: _helpers.coerceRootPath
      }
    }),
    handler: async argv => {
      (0, _cliHelpers.recordTelemetryAttributes)({
        command: 'serve',
        port: argv.port,
        host: argv.host,
        socket: argv.socket,
        apiRootPath: argv.apiRootPath
      });

      // Run the server file, if it exists, api side only
      if (hasServerFile()) {
        const {
          apiServerFileHandler
        } = await import('./serveApiHandler.js');
        await apiServerFileHandler(argv);
      } else {
        const {
          apiServerHandler
        } = await import('./serveApiHandler.js');
        await apiServerHandler(argv);
      }
    }
  }).command({
    command: 'web',
    description: webServerCLIConfig.description,
    builder: webServerCLIConfig.builder,
    handler: async argv => {
      (0, _cliHelpers.recordTelemetryAttributes)({
        command: 'serve',
        port: argv.port,
        host: argv.host,
        socket: argv.socket,
        apiHost: argv.apiHost
      });
      if ((0, _lib.getConfig)().experimental?.streamingSsr?.enabled) {
        await (0, _serveWebHandler.webSsrServerHandler)();
      } else {
        await webServerCLIConfig.handler(argv);
      }
    }
  }).middleware(argv => {
    (0, _cliHelpers.recordTelemetryAttributes)({
      command: 'serve'
    });

    // Make sure the relevant side has been built, before serving
    const positionalArgs = argv._;
    if ((0, _includes.default)(positionalArgs).call(positionalArgs, 'web') && !_fsExtra.default.existsSync(_path.default.join((0, _lib.getPaths)().web.dist), 'index.html')) {
      console.error(_colors.default.error('\n Please run `yarn rw build web` before trying to serve web. \n'));
      process.exit(1);
    }
    if ((0, _includes.default)(positionalArgs).call(positionalArgs, 'api') && !_fsExtra.default.existsSync(_path.default.join((0, _lib.getPaths)().api.dist))) {
      console.error(_colors.default.error('\n Please run `yarn rw build api` before trying to serve api. \n'));
      process.exit(1);
    }
    if (
    // serve both
    positionalArgs.length === 1 && (!_fsExtra.default.existsSync(_path.default.join((0, _lib.getPaths)().api.dist)) || !_fsExtra.default.existsSync(_path.default.join((0, _lib.getPaths)().web.dist), 'index.html'))) {
      console.error(_colors.default.error('\n Please run `yarn rw build` before trying to serve your redwood app. \n'));
      process.exit(1);
    }

    // Set NODE_ENV to production, if not set
    if (!process.env.NODE_ENV) {
      process.env.NODE_ENV = 'production';
    }
  }).epilogue(`Also see the ${(0, _terminalLink.default)('Redwood CLI Reference', 'https://redwoodjs.com/docs/cli-commands#serve')}`);
};
exports.builder = builder;