"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.apiServerHandler = exports.apiServerFileHandler = void 0;
var _now = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/date/now"));
var _path = _interopRequireDefault(require("path"));
var _chalk = _interopRequireDefault(require("chalk"));
var _execa = _interopRequireDefault(require("execa"));
var _fastify = require("@redwoodjs/fastify");
var _projectConfig = require("@redwoodjs/project-config");
const apiServerFileHandler = async argv => {
  await (0, _execa.default)('yarn', ['node', _path.default.join('dist', 'server.js'), '--port', argv.port, '--apiRootPath', argv.apiRootPath], {
    cwd: (0, _projectConfig.getPaths)().api.base,
    stdio: 'inherit'
  });
};
exports.apiServerFileHandler = apiServerFileHandler;
const apiServerHandler = async options => {
  const {
    port,
    socket,
    apiRootPath
  } = options;
  const tsApiServer = (0, _now.default)();
  console.log(_chalk.default.dim.italic('Starting API Server...'));
  const fastify = (0, _fastify.createFastifyInstance)();
  process.on('exit', () => {
    fastify?.close();
  });
  await fastify.register(_fastify.redwoodFastifyAPI, {
    redwood: {
      ...options
    }
  });
  let listenOptions;
  if (socket) {
    listenOptions = {
      path: socket
    };
  } else {
    listenOptions = {
      port,
      host: process.env.NODE_ENV === 'production' ? '0.0.0.0' : '::'
    };
  }
  const address = await fastify.listen(listenOptions);
  fastify.ready(() => {
    fastify.log.trace({
      custom: {
        ...fastify.initialConfig
      }
    }, 'Fastify server configuration');
    fastify.log.trace(`Registered plugins \n${fastify.printPlugins()}`);
    console.log(_chalk.default.dim.italic('Took ' + ((0, _now.default)() - tsApiServer) + ' ms'));
    const apiServer = _chalk.default.magenta(`${address}${apiRootPath}`);
    const graphqlEndpoint = _chalk.default.magenta(`${apiServer}graphql`);
    console.log(`API server listening at ${apiServer}`);
    console.log(`GraphQL endpoint at ${graphqlEndpoint}`);
    sendProcessReady();
  });
};
exports.apiServerHandler = apiServerHandler;
function sendProcessReady() {
  return process.send && process.send('ready');
}