"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js/object/define-property");
var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault").default;
_Object$defineProperty(exports, "__esModule", {
  value: true
});
exports.bothSsrRscServerHandler = exports.bothServerHandler = exports.bothServerFileHandler = void 0;
var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/promise"));
var _now = _interopRequireDefault(require("@babel/runtime-corejs3/core-js/date/now"));
var _path = _interopRequireDefault(require("path"));
var _chalk = _interopRequireDefault(require("chalk"));
var _concurrently = _interopRequireDefault(require("concurrently"));
var _execa = _interopRequireDefault(require("execa"));
var _fastify = require("@redwoodjs/fastify");
var _fastifyWeb = require("@redwoodjs/fastify-web");
var _projectConfig = require("@redwoodjs/project-config");
var _telemetry = require("@redwoodjs/telemetry");
var _exit = require("../lib/exit");
const bothServerFileHandler = async argv => {
  if ((0, _projectConfig.getConfig)().experimental?.rsc?.enabled || (0, _projectConfig.getConfig)().experimental?.streamingSsr?.enabled) {
    logSkippingFastifyWebServer();
    await (0, _execa.default)('yarn', ['rw-serve-fe'], {
      cwd: (0, _projectConfig.getPaths)().web.base,
      stdio: 'inherit',
      shell: true
    });
  } else {
    const apiHost = process.env.NODE_ENV === 'production' ? '0.0.0.0' : '::';
    const apiProxyTarget = `http://${apiHost}:${argv.apiPort}`;
    const {
      result
    } = (0, _concurrently.default)([{
      name: 'api',
      command: `yarn node ${_path.default.join('dist', 'server.js')} --port ${argv.apiPort}`,
      cwd: (0, _projectConfig.getPaths)().api.base,
      prefixColor: 'cyan'
    }, {
      name: 'web',
      command: `yarn rw-web-server --port ${argv.webPort} --api-proxy-target ${apiProxyTarget}`,
      cwd: (0, _projectConfig.getPaths)().base,
      prefixColor: 'blue'
    }], {
      prefix: '{name} |',
      timestampFormat: 'HH:mm:ss',
      handleInput: true
    });
    try {
      await result;
    } catch (error) {
      if (typeof error?.message !== 'undefined') {
        (0, _telemetry.errorTelemetry)(process.argv, `Error concurrently starting sides: ${error.message}`);
        (0, _exit.exitWithError)(error);
      }
    }
  }
};
exports.bothServerFileHandler = bothServerFileHandler;
const bothSsrRscServerHandler = async argv => {
  const {
    apiServerHandler
  } = await import('./serveApiHandler.js');

  // TODO Allow specifying port, socket and apiRootPath
  const apiPromise = apiServerHandler({
    ...argv,
    port: 8911,
    apiRootPath: '/'
  });

  // TODO More gracefully handle Ctrl-C
  // Right now you get a big red error box when you kill the process
  const fePromise = (0, _execa.default)('yarn', ['rw-serve-fe'], {
    cwd: (0, _projectConfig.getPaths)().web.base,
    stdio: 'inherit',
    shell: true
  });
  await _promise.default.all([apiPromise, fePromise]);
};
exports.bothSsrRscServerHandler = bothSsrRscServerHandler;
const bothServerHandler = async options => {
  const {
    port,
    socket
  } = options;
  const tsServer = (0, _now.default)();
  console.log(_chalk.default.italic.dim('Starting API and Web Servers...'));
  const fastify = (0, _fastify.createFastifyInstance)();
  process.on('exit', () => {
    fastify?.close();
  });
  await fastify.register(_fastifyWeb.redwoodFastifyWeb, {
    redwood: {
      ...options
    }
  });
  const apiRootPath = (0, _fastifyWeb.coerceRootPath)((0, _projectConfig.getConfig)().web.apiUrl);
  await fastify.register(_fastify.redwoodFastifyAPI, {
    redwood: {
      ...options,
      apiRootPath
    }
  });
  let listenOptions;
  if (socket) {
    listenOptions = {
      path: socket
    };
  } else {
    listenOptions = {
      port,
      host: process.env.NODE_ENV === 'production' ? '0.0.0.0' : '::'
    };
  }
  const address = await fastify.listen(listenOptions);
  fastify.ready(() => {
    console.log(_chalk.default.dim.italic('Took ' + ((0, _now.default)() - tsServer) + ' ms'));
    const webServer = _chalk.default.green(address);
    const apiServer = _chalk.default.magenta(`${address}${apiRootPath}`);
    const graphqlEndpoint = _chalk.default.magenta(`${apiServer}graphql`);
    console.log(`Web server listening at ${webServer}`);
    console.log(`API server listening at ${apiServer}`);
    console.log(`GraphQL endpoint at ${graphqlEndpoint}`);
    sendProcessReady();
  });
};
exports.bothServerHandler = bothServerHandler;
function sendProcessReady() {
  return process.send && process.send('ready');
}
function logSkippingFastifyWebServer() {
  console.warn('');
  console.warn('⚠️ Skipping Fastify web server ⚠️');
  console.warn('⚠️ Using new RSC server instead ⚠️');
  console.warn('');
}